import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  max,
  axisLeft,
  axisBottom,
  area,
  curveBasis,
} from 'd3';

// Select the SVG element from the DOM
const svg = select('svg');

// Get the width and height of the SVG element
const width = +svg.attr('width');
const height = +svg.attr('height');

// Function to render the area chart
const render = (data) => {
  // Title for the chart
  const title = 'Textile Sales Area Chart';

  // Function to get x-value from data
  const xValue = (d) => d.year;
  const xAxisLabel = 'Year'; // Label for x-axis

  // Function to get y-value from data
  const yValue = (d) => d.TotalSales;
  const yAxisLabel = 'Total Sales'; // Label for y-axis

  // Circle radius for data points
  const circleRadius = 6;

  // Margin object to provide spacing around the chart
  const margin = {
    top: 60,
    right: 40,
    bottom: 88,
    left: 185,
  };
  // Calculate inner width and height
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // Create x-scale using scaleTime
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  // Create y-scale using scaleLinear
  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0])
    .nice();

  // Append a 'g' element for positioning the chart content
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // Create x-axis
  const xAxis = axisBottom(xScale)
    .ticks(10)
    .tickSize(-innerHeight)
    .tickPadding(15);

  // Create y-axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  // Append y-axis to the chart
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -120)
    .attr('x', -innerHeight / 2)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);

  // Append x-axis to the chart
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  xAxisG.select('.domain').remove();
  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 80)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabel);

  // Create area generator
  const areaGenerator = area()
    .x((d) => xScale(xValue(d)))
    .y0(innerHeight)
    .y1((d) => yScale(yValue(d)))
    .curve(curveBasis);

  // Append path element for the area
  g.append('path')
    .attr('class', 'area-path')
    .attr('d', areaGenerator(data));

  // Append circles for each data point
  g.selectAll('.dot')
    .data(data)
    .enter()
    .append('circle')
    .attr('class', 'dot')
    .attr('cx', (d) => xScale(xValue(d)))
    .attr('cy', (d) => yScale(yValue(d)))
    .attr('r', circleRadius);

  // Append title for the chart
  svg
    .append('text')
    .attr('class', 'title')
    .attr('x', width / 2)
    .attr('y', 45)
    .text(title);
};

// Fetch data from CSV file
csv(
  'https://gist.githubusercontent.com/akhilcharugulla/d314cfc95b0d87ecc59768a5d8c9a018/raw/6beb4dd16209bb79a385d3bc359d2000f422086c/textile.csv',
).then((data) => {
  // Convert TotalSales to numeric and Year to Date object
  data.forEach((d) => {
    d.TotalSales = +d.TotalSales;
    d.year = new Date(d.Year);
  });
  // Call the render function with the fetched data
  render(data);
});
